﻿using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Forms.Integration;

namespace Roesberg.PDF.WPFDemoApp {
  public partial class MainWindow : Window {

	private List<PDFUserControl> openPDFUserControls = new List<PDFUserControl>();
	private string _applicationStartupPath;
    private string _sampleDocumentsPath;
    private int _lastOpenedNumber = 0;

	private PDFXEdit.PXV_Inst pdfInst;


	public MainWindow()
	{
		InitializeComponent();
		pdfInst = new PDFXEdit.PXV_Inst();
		pdfInst.Init();

		_applicationStartupPath = System.IO.Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
		_sampleDocumentsPath = _applicationStartupPath + @"\..\..\SamplePDFs";

		Closed += OnWindowClosed;
    }

	private void OnWindowClosed(object sender, System.EventArgs e)
	{
		/////////////////////////////////////////////////////////////////////////////////////
		// Forced release of all COM-objects that may still captured by Garbage Collector. //
		// It is critical to release them before destroying of pdfInst                     //
		/////////////////////////////////////////////////////////////////////////////////////

		System.GC.Collect();
		System.GC.WaitForPendingFinalizers();

		pdfInst.Shutdown();
		pdfInst = null;
	}
		
	private void OpenMulti10Button_Click(object sender, RoutedEventArgs e) {
      OpenDocuments(10);
    }

    private void OpenMulti20Button_Click(object sender, RoutedEventArgs e) {
      OpenDocuments(20);
    }

    private void OpenDocuments(int numberOfDocuments) {
      CloseAllOpenDocuments();
      _lastOpenedNumber = 0;

      try {
        Stopwatch sw = new Stopwatch();
        sw.Start();

        for (int i = 1; i <= numberOfDocuments; i++) {
          OpenDocumentInNewTab(i);
          _lastOpenedNumber++;
        }

        Debug.WriteLine(@"Elapsed millisec. for {numberOfDocuments} documents: {sw.ElapsedMilliseconds}");
        ElapsedTimeTextBlock.Text = sw.ElapsedMilliseconds.ToString();
        sw.Stop();
      }
      catch (FileNotFoundException fileEx) {
        MessageBox.Show(fileEx.Message + "\n" + fileEx.FileName, "File not found");
      }
    }

    private void OpenOneMoreButton_Click(object sender, RoutedEventArgs e) {
      OpenDocumentInNewTab(++_lastOpenedNumber);
    }

    private void Open10MoreButton_Click(object sender, RoutedEventArgs e) {
      Stopwatch sw = new Stopwatch();
      sw.Start();

      for (int i = 1; i <= 10; i++) {
        OpenDocumentInNewTab(++_lastOpenedNumber);
      }

      ElapsedTimeTextBlock.Text = sw.ElapsedMilliseconds.ToString();
      sw.Stop();
    }

    private void OpenDocumentInNewTab(int i) {
      var pathToDocument = string.Format("{0}\\{1}.pdf", _sampleDocumentsPath, i);

      if (!File.Exists(pathToDocument)) {
        throw new FileNotFoundException("File not found", pathToDocument);
      }

      var pdfUserControl = new PDFUserControl();

      openPDFUserControls.Add(pdfUserControl);

      TabItem tp = new TabItem();
      tp.Header = string.Format("{0}.pdf", i);

      var formsHost = new WindowsFormsHost();
      formsHost.Child = pdfUserControl;

	  tp.Content = formsHost;
      MainTab.Items.Add(tp);

      pdfUserControl.OpenDocument(pathToDocument);

      if (i == 1) {
        pdfUserControl.EnableToolbars();
      }

      OpenDocumentsTextBlock.Text = openPDFUserControls.Count.ToString();
    }

    private void CloseAllButton_Click(object sender, RoutedEventArgs e) {
      CloseAllOpenDocuments();
    }

    private void CloseAllOpenDocuments()
	{
		if (openPDFUserControls.Count == 0)
			return;

		foreach (var userControl in openPDFUserControls)
			userControl.Dispose();

		openPDFUserControls.Clear();

		MainTab.Items.Clear();
    }

    private void ShowVersionButton_Click(object sender, RoutedEventArgs e) {
      if (openPDFUserControls.Count >= 1) {
        openPDFUserControls[0].ShowVersion();
      }
    }
  }
}
